// ========================================================
//
// CBP_2403_R_Gap_Navigation
// 29-09-21
// Based on CBP_ePuck_ObjDet2a.c (2020-21 2403)
//
// ========================================================

#include <Servo.h>

// Define pins ---------------
#define HC_SR04_Tx 2
#define HC_SR04_Rx 3
#define ServoL_pin 13
#define ServoR_pin 12
#define ServoTurret_pin 11

// Define constants ----------
#define dT 20
#define DEBUG true
#define wheelDiam 66.0
#define wheelRad (wheelDiam/2.0)
#define axleLen 104.0
#define distThreshold 300

// Global variables ----------
Servo servoL;
Servo servoR; 
Servo servoTurret;  

float driveL, driveR;
float dist;

int angle = 0;  
int totalAngle = 0;
int avAngle;
int count;

float rotAngle;
int labelArray[180];
int index;

#define STATE_CENTRE_TURRET 1
#define STATE_CALIBRATE_ROTATE 2

#define STATE_SCANNING  3
#define STATE_ANALYSING 4
#define STATE_ROTATE_TO_GAP 5
#define STATE_DONE 6

int state;

void setup() {
  Serial.begin(9600);
  cprintf(">>> Initializing stuff ..................\n"); 
  pinMode(HC_SR04_Tx,OUTPUT);
  pinMode(HC_SR04_Rx,INPUT);
  servoTurret.attach(ServoTurret_pin);
  servoL.attach(ServoL_pin);                         
  servoR.attach(ServoR_pin);  
  servoL.writeMicroseconds(1500); 
  servoR.writeMicroseconds(1500);
  servoTurret.write(90); 

  cprintf(">>> Pausing 2 secs ......................\n");
  delay(2000);
  cprintf(">>> Starting  ...........................\n"); 

  //cprintf(">>> Transit to STATE_CENTRE_TURRET ......\n"); 
  //state = STATE_CENTRE_TURRET;
  //cprintf(">>> Transit to STATE_CALIBRATE_ROTATE ...\n"); 
  //state = STATE_CALIBRATE_ROTATE;
  cprintf(">>> Transit to STATE_SCANNING ...........\n"); 
  state = STATE_SCANNING;  

}


void loop() {
  switch(state) {
     case STATE_CENTRE_TURRET:
       servoTurret.write(90);
       state = STATE_DONE;
       break;

     case STATE_CALIBRATE_ROTATE:
       rotateByDegrees(90);
       state = STATE_DONE;
       break;
              
     case STATE_SCANNING:
      for (angle = 20; angle <= 160; angle += 1) { 
        servoTurret.write(angle);  
        // Add code here ------------------------------------          



        delay(60);  
        if(DEBUG)cprintf("label = %d\n",labelArray[angle]);                    
      }
      for (angle = 20; angle <= 160; angle += 1) {
        cprintf("angle = %d, label = %d\n",angle,labelArray[angle]);
      }
      cprintf(">>> Transit to STATE_DONE ...........\n");      
      state = STATE_DONE;
      //cprintf(">>> Transit to STATE_ANALYZING.......\n");      
      //state = STATE_ANALYSING;
      break;
      
     case STATE_ANALYSING:
       for (angle = 20; angle <= 160; angle +=1) {
         // Add code here -----------------------------




       }
       // Add code here --------------------------------
       
       cprintf("avAngle = %i\n",avAngle);
       
      // cprintf("Min dist = %f at angle = %d\n",minDist,angle);

       //cprintf(">>> Transit to STATE_DONE ...........\n");      
       //state = STATE_DONE;
       cprintf(">>> Transit to STATE_ROTATE_TO_GAP\n");      
       state = STATE_ROTATE_TO_GAP;
       break;
      
     case STATE_ROTATE_TO_GAP :

       //Add code here ---------------------------------
       
       state = STATE_DONE;
       break;      
      
      case STATE_DONE:
        driveL=0;
        driveR=0;
        driveServos(driveL,driveR);  
        servoTurret.write(90);
        delay(500);
        servoL.detach();
        servoR.detach();
        servoTurret.detach();      
        break;
  }
}

void rotateByDegrees(float deg) {
  int direction;
  float driveTime;
  float correctionFactor;
  direction = deg/abs(deg);
  driveR = 20*direction;
  driveL = -driveR;
  correctionFactor = 1.5;
  driveTime = correctionFactor*deg*1654.0/90.0;
  driveServos(driveL,driveR);
  if(DEBUG)
    cprintf("driveTime = %f\n",driveTime);
  delay((int)(driveTime));  
}

float getDistance() {
  long duration;
  float mm;
  digitalWrite(HC_SR04_Tx, LOW);
  delayMicroseconds(2);
  digitalWrite(HC_SR04_Tx, HIGH);
  delayMicroseconds(10);
  digitalWrite(HC_SR04_Tx, LOW);

  duration = pulseIn(HC_SR04_Rx, HIGH);
  mm = 10*duration / 29 / 2;
  
  // Catch 'bad' values and report as mm = -1;
  if(duration < 600) mm = -1;
  if(mm > 4000) mm = -1;
  
  //cprintf("Duration (muSec) = %l, dist (mm) = %f\n",duration,mm);
  return mm;
  
}

void driveServos(float _driveL, float _driveR) {
  servoL.writeMicroseconds(1500 + (int)_driveL);
  servoR.writeMicroseconds(1500 - (int)_driveR);
}
