// ======================================================
// CBP_2403_R_LocateBlock
// 23-10-21
// Locates a block outside the FOV by scanning
// Uses LUT to convert width to distance
// FBO_CBP_MoveToBlock.ino
// FB0 CBP 06-07-21
// ======================================================

#include <Pixy2.h>
#include <Servo.h>

// Define pins ---------------
#define ServoL_pin 13
#define ServoR_pin 12
#define ServoTurret_pin 11

// Define constants ----------
#define DEBUG true
#define wheelDiam 66.0
#define wheelRad (wheelDiam/2.0)
#define axleLen 104.0

// Global variables ----------
Servo servoL;
Servo servoR; 
Servo servoTurret;

int offset;
float driveL, driveR;
int angle;
int angleFound;  
float rotAngle;
//float distArray[180];

// Pixy variables -----------------------
// This is the main Pixy object 
Pixy2 pixy;
uint16_t width;
uint16_t height;
uint16_t color;
uint8_t index;
uint16_t x;    // Must be 16 else caps at 255
uint16_t y;
uint16_t frameWidth;

// controller variables -----------------
float error;
float errorThresh;

// FSM ----------------------------------
#define STATE_SCANNING  1
#define STATE_ROTATE_TO_OBJECT 2
#define STATE_DONE 3
int state;

void setup() {
  Serial.begin(9600);
  cprintf(">>> Initializing stuff ..................\n");
  Serial.begin(9600);
  Serial.print("Starting...\n");

  // Make sure camera is pointing forwards.  
  // Apply your offset here
  offset = 0;
  servoTurret.attach(ServoTurret_pin);
  // Now set it to 90 degs pointing forwards
  servoTurret.write(90 + offset);
  delay(1000);

  // Init the camera
  pixy.init();
  // get details of the camera
  frameWidth = pixy.frameWidth;

  // Attach the servos
  servoL.attach(ServoL_pin);
  servoR.attach(ServoR_pin);
  cprintf(">>> Pausing 2 secs ......................\n");
  delay(2000);
  cprintf(">>> Starting  ...........................\n"); 
  cprintf("frameWidth = %d\n",frameWidth);

  errorThresh = 0.1;

  state = STATE_SCANNING;
  
}

void loop() {
  float dist;
  float deriv;

  switch(state) {
    case STATE_SCANNING:
      for (angle = 20; angle <= 160; angle += 1) { 
        servoTurret.write(angle);  
        pixy.ccc.getBlocks(true, 1, 1);          
        if (pixy.ccc.numBlocks > 0) {
           // Add a line to get the x-coord of the blob
           
           // Add a line to calc the error
           
           // Add a line to normalize the error to range -1.0 to 1.0
           
           cprintf("Angle = %d, Error = %f\n",angle,error);
           // Add an if( ) to test the error is less than
           // errorThresh and if so, store the angle and break


                          
        }    
      }
      cprintf("Object found at angle %d\n",angleFound);
      state = STATE_ROTATE_TO_OBJECT;
      break;
    case STATE_ROTATE_TO_OBJECT:
      // Add code to get rotAngle
      
      rotateByDegrees(rotAngle);   
      state = STATE_DONE;    
      break;      
    case STATE_DONE:
      driveR=0;
      driveServos(driveL,driveR);  
      servoTurret.write(90);
      delay(500);
      servoL.detach();
      servoR.detach();
      servoTurret.detach();    
      break;
  }
}

float getDistanceFromObject(uint16_t _width) {
  float dist;
  dist = 15276.2/((float)_width) - 38.3;
  return dist;
}

void driveServos(float _driveL, float _driveR) {
  servoL.writeMicroseconds(1500 + (int)_driveL);
  servoR.writeMicroseconds(1500 - (int)_driveR);
}

void rotateByDegrees(float deg) {
  int direction;
  float driveTime;
  float correctionFactor;
  direction = deg/abs(deg);
  driveR = 20*direction;
  driveL = -driveR;
  correctionFactor = 1.0;
  driveTime = correctionFactor*abs(deg)*1654.0/90.0;
  driveServos(driveL,driveR);
  if(DEBUG)
    cprintf("driveTime = %f\n",driveTime);
  delay((int)(driveTime));  
}
